@@ -4,6 +4,7 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
+	"net/http"
 	"regexp"
 	"runtime"
 	"sync"
@@ -12,6 +13,7 @@ import (
 
 	"github.com/docker/distribution"
 	"github.com/docker/distribution/manifest/manifestlist"
+	"github.com/docker/distribution/manifest/ocischema"
 	"github.com/docker/distribution/manifest/schema1"
 	"github.com/docker/distribution/manifest/schema2"
 	"github.com/docker/distribution/reference"
@@ -21,8 +23,9 @@ import (
 	"github.com/docker/libtrust"
 	"github.com/opencontainers/go-digest"
 	"k8s.io/client-go/rest"
-	"k8s.io/klog"
+	"k8s.io/klog/v2"
 
+	imagespecv1 "github.com/opencontainers/image-spec/specs-go/v1"
 	"github.com/openshift/library-go/pkg/image/dockerv1client"
 	imagereference "github.com/openshift/library-go/pkg/image/reference"
 	"github.com/openshift/library-go/pkg/image/registryclient"
@@ -52,6 +55,23 @@ func (o *SecurityOptions) Bind(flags *pflag.FlagSet) {
 	flags.BoolVar(&o.SkipVerification, "skip-verification", o.SkipVerification, "Skip verifying the integrity of the retrieved content. This is not recommended, but may be necessary when importing images from older image registries. Only bypass verification if the registry is known to be trustworthy.")
 }
 
+// ReferentialHTTPClient returns an http.Client that is appropriate for accessing
+// blobs referenced outside of the registry (due to the present of the URLs attribute
+// in the manifest reference for a layer).
+func (o *SecurityOptions) ReferentialHTTPClient() (*http.Client, error) {
+	ctx, err := o.Context()
+	if err != nil {
+		return nil, err
+	}
+	client := &http.Client{}
+	if o.Insecure {
+		client.Transport = ctx.InsecureTransport
+	} else {
+		client.Transport = ctx.Transport
+	}
+	return client, nil
+}
+
 type Verifier interface {
 	Verify(dgst, contentDgst digest.Digest)
 	Verified() bool
@@ -193,6 +213,7 @@ type FilterFunc func(*manifestlist.ManifestDescriptor, bool) bool
 var PreferManifestList = distribution.WithManifestMediaTypes([]string{
 	manifestlist.MediaTypeManifestList,
 	schema2.MediaTypeManifest,
+	imagespecv1.MediaTypeImageManifest,
 })
 
 // AllManifests returns all non-list manifests, the list manifest (if any), the digest the from refers to, or an error.
@@ -301,8 +322,31 @@ func ManifestToImageConfig(ctx context.Context, srcManifest distribution.Manifes
 
 		return base, layers, nil
 
+	case *ocischema.DeserializedManifest:
+		if t.Config.MediaType != imagespecv1.MediaTypeImageConfig {
+			return nil, nil, fmt.Errorf("%s does not have the expected image configuration media type: %s", location, t.Config.MediaType)
+		}
+		configJSON, err := blobs.Get(ctx, t.Config.Digest)
+		if err != nil {
+			return nil, nil, fmt.Errorf("cannot retrieve image configuration for %s: %v", location, err)
+		}
+		klog.V(4).Infof("Raw image config json:\n%s", string(configJSON))
+		config := &dockerv1client.DockerImageConfig{}
+		if err := json.Unmarshal(configJSON, &config); err != nil {
+			return nil, nil, fmt.Errorf("unable to parse image configuration: %v", err)
+		}
+
+		base := config
+		layers := t.Layers
+		base.Size = 0
+		for _, layer := range t.Layers {
+			base.Size += layer.Size
+		}
+
+		return base, layers, nil
+
 	case *schema1.SignedManifest:
-		if klog.V(4) {
+		if klog.V(4).Enabled() {
 			_, configJSON, _ := srcManifest.Payload()
 			klog.Infof("Raw image config json:\n%s", string(configJSON))
 		}
@@ -349,10 +393,10 @@ func ProcessManifestList(ctx context.Context, srcDigest digest.Digest, srcManife
 		filtered := make([]manifestlist.ManifestDescriptor, 0, len(t.Manifests))
 		for _, manifest := range t.Manifests {
 			if !filterFn(&manifest, len(t.Manifests) > 1) {
-				klog.V(5).Infof("Skipping image for %#v from %s", manifest.Platform, ref)
+				klog.V(5).Infof("Skipping image %s for %#v from %s", manifest.Digest, manifest.Platform, ref)
 				continue
 			}
-			klog.V(5).Infof("Including image for %#v from %s", manifest.Platform, ref)
+			klog.V(5).Infof("Including image %s for %#v from %s", manifest.Digest, manifest.Platform, ref)
 			filtered = append(filtered, manifest)
 		}
 
@@ -393,7 +437,7 @@ func ProcessManifestList(ctx context.Context, srcDigest digest.Digest, srcManife
 			if err != nil {
 				return nil, nil, "", err
 			}
-			klog.V(5).Infof("Used only one manifest from the list %s", srcDigest)
+			klog.Warningf("Chose %s/%s manifest from the manifest list.", t.Manifests[0].Platform.OS, t.Manifests[0].Platform.Architecture)
 			return srcManifests, srcManifests[0], manifestDigest, nil
 		default:
 			return append(srcManifests, manifestList), manifestList, manifestDigest, nil
@@ -428,7 +472,7 @@ func ManifestsFromList(ctx context.Context, srcDigest digest.Digest, srcManifest
 	}
 }
 
-// TDOO: remove when quay.io switches to v2 schema
+// TODO: Remove support for v2 schema in 4.9
 func PutManifestInCompatibleSchema(
 	ctx context.Context,
 	srcManifest distribution.Manifest,
@@ -452,6 +496,7 @@ func PutManifestInCompatibleSchema(
 			klog.V(2).Infof("Unable to convert manifest to schema2: %v", err)
 			return toManifests.Put(ctx, t, distribution.WithTag(tag))
 		}
+		klog.Infof("warning: Digests are not preserved with schema version 1 images. Support for schema version 1 images will be removed in a future release")
 		return toManifests.Put(ctx, manifest, options...)
 	}
 
@@ -479,14 +524,14 @@ func PutManifestInCompatibleSchema(
 	klog.V(5).Infof("Registry reported invalid manifest error, attempting to convert to v2schema1 as ref %s", tagRef)
 	schema1Manifest, convertErr := convertToSchema1(ctx, blobs, configJSON, schema2Manifest, tagRef)
 	if convertErr != nil {
-		if klog.V(6) {
+		if klog.V(6).Enabled() {
 			_, data, _ := schema2Manifest.Payload()
 			klog.Infof("Input schema\n%s", string(data))
 		}
 		klog.V(2).Infof("Unable to convert manifest to schema1: %v", convertErr)
 		return toDigest, err
 	}
-	if klog.V(6) {
+	if klog.V(6).Enabled() {
 		_, data, _ := schema1Manifest.Payload()
 		klog.Infof("Converted to v2schema1\n%s", string(data))
 	}
@@ -496,7 +541,7 @@ func PutManifestInCompatibleSchema(
 // convertToSchema2 attempts to build a v2 manifest from a v1 manifest, which requires reading blobs to get layer sizes.
 // Requires the destination layers already exist in the target repository.
 func convertToSchema2(ctx context.Context, blobs distribution.BlobService, srcManifest *schema1.SignedManifest) (distribution.Manifest, error) {
-	if klog.V(6) {
+	if klog.V(6).Enabled() {
 		klog.Infof("Up converting v1 schema image:\n%#v", srcManifest.Manifest)
 	}
 
@@ -504,7 +549,7 @@ func convertToSchema2(ctx context.Context, blobs distribution.BlobService, srcMa
 	if err != nil {
 		return nil, err
 	}
-	if klog.V(6) {
+	if klog.V(6).Enabled() {
 		klog.Infof("Resulting schema: %#v", config)
 	}
 	// create synthetic history
@@ -521,7 +566,7 @@ func convertToSchema2(ctx context.Context, blobs distribution.BlobService, srcMa
 	if err != nil {
 		return nil, err
 	}
-	if klog.V(6) {
+	if klog.V(6).Enabled() {
 		klog.Infof("Resulting config.json:\n%s", string(configJSON))
 	}
 	b := schema2.NewManifestBuilder(blobs, schema2.MediaTypeImageConfig, configJSON)
@@ -538,7 +583,7 @@ func convertToSchema2(ctx context.Context, blobs distribution.BlobService, srcMa
 	return b.Build(ctx)
 }
 
-// TDOO: remove when quay.io switches to v2 schema
+// TODO: Remove support for v2 schema in 4.9
 func convertToSchema1(ctx context.Context, blobs distribution.BlobService, configJSON []byte, schema2Manifest *schema2.DeserializedManifest, ref reference.Named) (distribution.Manifest, error) {
 	if configJSON == nil {
 		targetDescriptor := schema2Manifest.Target()
@@ -552,7 +597,7 @@ func convertToSchema1(ctx context.Context, blobs distribution.BlobService, confi
 	if err != nil {
 		return nil, err
 	}
-	if klog.V(6) {
+	if klog.V(6).Enabled() {
 		klog.Infof("Down converting v2 schema image:\n%#v\n%s", schema2Manifest.Layers, configJSON)
 	}
 	builder := schema1.NewConfigManifestBuilder(blobs, trustKey, ref, configJSON)
@@ -573,7 +618,7 @@ var (
 	privateKey     libtrust.PrivateKey
 )
 
-// TDOO: remove when quay.io switches to v2 schema
+// TODO: Remove support for v2 schema in 4.9
 func loadPrivateKey() (libtrust.PrivateKey, error) {
 	privateKeyLock.Lock()
 	defer privateKeyLock.Unlock()
